<?php

class Ext_Form
{
  protected $id;
  protected $c;
  protected $options;
  protected $dataIn;
  protected $colInfo;
  protected $title_font;
  protected $title_font_size;
  protected $axis_font;
  protected $axis_font_size;
  protected $json;
  protected $json_parameter_name;
  protected $x;
  protected $y;

  function __construct($id, $options, $dataIn, $colInfo)
  {
    global $mydbr_defaults;

    $this->id = $id;
    $this->options = $options['dbr.form'];
    $this->dataIn = $dataIn;
    $this->colInfo = $colInfo;
    $this->json = false;
    $this->json_parameter_name = $mydbr_defaults['form_extension']['json_parameter_name'];
  }

  function template_variable($var)
  {
    if (_str_starts_with($var, '{#') && _str_ends_with($var, '}')) {
      $var = _substr($var, 2, _strlen($var) - 3);

      if ($var = Extension::record_get($var)) {
        if (_str_starts_with($var, '<div class="form-field"><div class="form-field-control"><select')) {
          $var = _substr($var, 56, _strlen($var) - 68);
        }
        return $var;
      }
    }
    return '';
  }

  private function getNameRequired($n)
  {
    $name = 'in_' . $n;
    $param = Extension::form_params_get($name);
    $required = ($param && !$param['optional']) ? ' required' : '';

    return ['name' => $name, 'required' => $required];
  }

  private function getDefault($n, $opt)
  {
    $ind = array_search($n, $this->options['name']);
    return isset($this->options[$opt][$ind]) ? $this->options[$opt][$ind] : null;
  }

  function process()
  {
    $actions_a = $this->options['action'];

    foreach ($actions_a as $action_ind => $actions) {
      foreach (explode(',', (string)$actions) as $key => $action) {
        switch ($action) {
          case 'start':

            $n = $this->options['name'][$action_ind];
            $action = '';
            for ($i = 1; $i <= sizeof($this->dataIn[0]); $i++) {
              $dr = Extension::getFormLinkParams($this->dataIn[0], $i);
              if ($dr[0]) {
                $action = $dr[0];
                Extension::form_params_set($dr[1]);
              }
            }
            echo '<form action="' . $action . '" method="post" id="' . $n . '" accept-charset="utf-8" '
              . ($this->json ? 'data-json="1" data-jsonparam="' . $this->json_parameter_name . '" ' : '') . ' data-formext="1"><div class="form-all">' .
              '<ul class="form-section page-section">';
            break;
          case 'json':
            $this->json = true;
            if (isset($this->options['name'][$action_ind])) {
              $this->json_parameter_name = $this->options['name'][$action_ind];
            }
            break;
          case 'end':
            echo '</ul></div></form>';
            break;
          case 'header.start':
            echo '<div class="q-header">';
            break;
          case 'header.end':
            echo '</div>';
            break;
          case 'section.start':
          case 'section.start.no_highlite':
            $datahighlite = ($action == 'section.start.no_highlite') ? ' data-highlite="no"' : '';
            echo '<li class="form-line"' . $datahighlite . '><div class="form-input-wide">';
            break;
          case 'section.end':
            echo '</div></li>';
            break;
          case 'radio':
            if (isset($this->options['text'])) {
              $t = $this->options['text'][$action_ind];
              echo '<label class="form-label form-label-top form-label-auto">' . pl($t) . '</label>';
            };
            $n = $this->options['name'][$action_ind];
            echo '<div class="form-single-column">';
            $nr = $this->getNameRequired($n);
            $def = $this->getDefault($n, 'opt0');
            foreach ($this->dataIn as $key => $data) {
              echo '<span class="form-radio-item" style="clear:left">
                  <input type="radio" ' . ((!is_null($def) && $def == $key) ? 'checked ' : '') . 'class="form-radio" id="' . $n . $key . '" name="' . $nr['name'] . '" value="' . $key . '"' . $nr['required'] . '>
                  <label for="' . $n . $key . '">' . pl($data[1]) . '</label>';
              if (isset($data[2])) {
                echo $this->template_variable($data[2]);
              }
              echo '</span>';
            }
            echo '</div>';
            break;
          case 'select':
            $skip_div = isset($this->options['opt0'][$action_ind]) && $this->options['opt0'][$action_ind] == 'plain';
            $n = $this->options['name'][$action_ind];
            $t = $this->options['text'][$action_ind];
            $nr = $this->getNameRequired($n);
            $def = $this->getDefault($n, 'opt1');
            if (isset($this->options['opt0'][$action_ind]) && $this->options['opt0'][$action_ind] == 'required') {
              $nr['required'] = ' required';
            }
            if (!$skip_div) {
              echo '<div class="form-field">';
              if ($t) {
                echo '<label class="form-field-label" for="' . $n . '">' . pl($t) . '</label>';
              };
              echo '<div class="form-field-control">';
            }
            echo '<select class="form-dropdown inline" data-required=' . pl_attr(trim($nr['required'])) . ' id="' . $n . '" name="' . $nr['name'] . '"' . $nr['required'] . '>';
            foreach ($this->dataIn as $key => $data) {
              echo '<option ' . ($def == $data[0] ? 'selected ' : '') . 'value="' . $data[0] . '">' . $data[1] . '</option>';
            }
            echo '</select>';
            if (!$skip_div) {
              echo '</div></div>';
            }
            break;
          case 'table':
            $n = $this->options['name'][$action_ind];
            if (isset($this->options['text'])) {
              $t = $this->options['text'][$action_ind];
              echo '<label class="form-field-label" for="' . $n . '">' . pl($t) . '</label>';
            };
            $cols = sizeof($this->options) - 3;
            echo '<div class="form-input-wide"><table cellpadding="4" cellspacing="0" class="form-matrix-table">
              <tbody>
                <tr class="form-matrix-tr form-matrix-header-tr">
                  <th class="form-matrix-th" style="border:none">&nbsp;</th>';
            for ($i = 0; $i < $cols; $i++) {
              $label = $this->options['opt' . $i][$action_ind];
              echo '<th class="form-matrix-headers form-matrix-column-headers form-matrix-column_' . $i . '"><label>' . $label . '</label></th>';
            }
            echo '</tr>';
            foreach ($this->dataIn as $key => $data) {
              echo '<tr class="form-matrix-tr form-matrix-value-tr">';
              echo '<th scope="row" class="form-matrix-headers form-matrix-row-headers' . ($key == 0 ? ' top' : '') . '"><label>' . pl($data[1]) . '</label></th>';
              $nr = $this->getNameRequired($data[0]);
              for ($i = $cols; $i > 0; $i--) {

                echo '<td class="form-matrix-values">' .
                  '<input type="radio" ' . ((isset($data[2]) && $data[2] == $i) ? 'checked ' : '') . 'id="' . $data[0] . $i . '" class="form-radio" name="' . $nr['name'] . '" value="' . $i . '"' . $nr['required'] . '><label for="' . $data[0] . $i . '" class="matrix-choice-label matrix-radio-label"> </label></td>';
              }
              echo '</tr>';
            }

            echo '</tbody></table></div>';

            break;
          case 'checkbox':
            $n = $this->options['name'];
            echo '<div class="form-field">';
            if (isset($this->options['text'])) {
              $t = $this->options['text'][$action_ind];
              echo '<label class="form-label form-label-top form-label-auto space_above">' . pl($t) . '</label>';
            };
            if (is_array($n)) {
              $cols = $n[$action_ind];
            } else {
              $cols = intval($n);
            }
            if ($cols > 1) {
              echo '<div class="form-input-wide"><div class="form-multiple-column" data-cols=' . pl_attr($cols) . '>';
              $style = "";
            } else {
              echo '<div class="form-single-column">';
              $style = ' style="clear:left;"';
            }
            foreach ($this->dataIn as $key => $data) {
              $nr = $this->getNameRequired($data[0]);
              echo '<span class="form-checkbox-item"' . $style . '>' .
                '<input type="checkbox" class="form-checkbox" id="' . $data[0] . '" name="' . $nr['name'] . '" ' . ((isset($data[2]) && $data[2]) ? 'checked ' : '') . 'value="1">' .
                '<label for="' . $data[0] . '">' . $data[1] . '</label>';
              if (isset($data[2])) {
                echo $this->template_variable($data[2]);
              }
              echo '</span>';
            }
            if ($cols > 1) {
              echo '</div>';
            }
            echo '</div></div>';

            break;
          case 'text':
          case 'date':
            $n = $this->options['name'][$action_ind];
            $nr = $this->getNameRequired($n);
            $def = $this->getDefault($n, 'opt2');
            $ph = $this->getDefault($n, 'opt3');
            if (isset($this->options['text'])) {
              $t = $this->options['text'][$action_ind];
              if ($t) {
                echo '<label class="form-label form-label-top form-label-auto space_above" for="' . $n . '">' . pl($t) . '</label>';
              }
            };
            $w = isset($this->options['opt0'][$action_ind]) ? intval($this->options['opt0'][$action_ind]) : 30;
            $size = ($w > 0) ? ' size="' . $w . '"' : '';
            if (isset($this->options['opt1'][$action_ind]) && $this->options['opt1'][$action_ind] == 'required') {
              $nr['required'] = ' required';
            }
            if ($action == 'date') {
              echo '<input type="date" class="form-input inline" value="' . htmlspecialchars((string)$def) . '" data-required=' . pl_attr(trim($nr['required'])) . ' id="' . $n . '" name="' . $nr['name'] . '"' . $size . $nr['required'] . '>';
            } else {
              echo '<input type="text" class="form-textbox inline" ' .
                ($ph ? 'placeholder="' . htmlspecialchars((string)$ph) . '" ' : '') .
                'value="' . htmlspecialchars((string)$def) . '" data-required=' . pl_attr(trim($nr['required'])) . ' id="' . $n . '" name="' . $nr['name'] . '"' . $size . $nr['required'] . '>';
            }
            break;
          case 'textarea':
            $n = $this->options['name'][$action_ind];
            $nr = $this->getNameRequired($n);
            $def = $this->getDefault($n, 'opt1');
            $ph = $this->getDefault($n, 'opt2');
            if (isset($this->options['text'])) {
              $t = $this->options['text'][$action_ind];
              if ($t) {
                echo '<label class="form-label form-label-top form-label-auto space_above" for="' . $n . '">' . pl($t) . '</label>';
              }
            };
            if (isset($this->options['opt0'][$action_ind]) && $this->options['opt0'][$action_ind] == 'required') {
              $nr['required'] = ' required';
            }
            echo '<textarea id="' . $n . '" ' .
              ($ph ? 'placeholder="' . htmlspecialchars((string)$ph) . '" ' : '') .
              'class="form-textarea" name="' . $nr['name'] . '"' . $nr['required'] . '>' . htmlspecialchars((string)$def) . '</textarea>';
            break;
          case 'submit':
            $n = $this->options['name'][$action_ind];
            echo '<li class="form-line" data-highlite="no">
                   <div class="form-input-wide">
                     <div class="form-buttons-wrapper form-buttons-auto">
                     <button type="submit" class="form-submit-button submit-button">' . $n . '</button>
                     </div>
                   </div>
                   </li>';
            break;
        }
      }
    }
  }
}


function Ext_Form($id, $options, $dataIn, $colInfo)
{
  $fb = new Ext_Form($id, $options, $dataIn, $colInfo);
  $fb->process();
}
