/*
  Copyright mydbr.com https://mydbr.com 2007-2024
  You are free to modify this file
*/

function initGoogleMap() {
  Google_maps.forEach((gmap) => {
    var map_type, latlng = new google.maps.LatLng(gmap.x, gmap.y);
    switch (gmap.type) {
      case 'satellite':
        map_type = google.maps.MapTypeId.SATELLITE;
        break;
      case 'terrain':
        map_type = google.maps.MapTypeId.TERRAIN;
        break;
      case 'hybrid':
        map_type = google.maps.MapTypeId.HYBRID;
        break;
      default:
        map_type = google.maps.MapTypeId.ROADMAP;
        break;
    }

    var options = {
      zoom: gmap.zoom,
      center: latlng,
      mapTypeControl: true,
      mapTypeControlOptions: {style: google.maps.MapTypeControlStyle.DROPDOWN_MENU},
      navigationControl: true,
      mapTypeId: map_type,
      mapId: 'myDBRmap'
    };
    
    gmap.map = new google.maps.Map(document.getElementById('map'+gmap.id), options);
    gmap.geoJSON.forEach((geoJSON) => {
      gmap.map.data.loadGeoJson( geoJSON ) ;
    });

    gmap.KML.forEach((kml) => {
      var kmll = new google.maps.KmlLayer({ url: kml, map: gmap.map });
    });

    if (!gmap.geoJSON_style) {
      gmap.geoJSON_style = {
        fillColor: '#98EE9E',
        strokeColor: '#939494',
        strokeWeight: 2,
      };
    }
    gmap.map.data.setStyle((feature) => {
      return gmap.geoJSON_style;
    });


    gmap.points = [];
    
    /* Normal markers */
    gmap.markers.forEach((m, i) => {
      gmap.markers_show(i);
    });

    if (gmap.is_route) {
      gmap.route_show();
    }

    gmap.markers = gmap.markers_extra;
    /* Extra markers */
    gmap.markers.forEach((m, i) => {
      gmap.markers_show(i);
    });
    
    if (gmap.polyline_options) {
      gmap.polyline_show();
    }
    gmap.heatmap_show();
    gmap.set_bounds_show();
    gmap.fix_zoom2();
    
  });
}

var Google_map = {
  id: null,
  map : null,
  x: null,
  y: null,
  type: null,
  is_route: null,
  points: [],
  markers: [],
  markers_extra: [],
  route_color: [],
  markers: [],
  is_route: false,
  map_type: 0,
  zoom: null,
  bounds: null,
  noXY: null, 
  noZoom: null, 
  polyline_options: null, 
  heatmap_locations: [],
  heatmap_points: [],
  geoJSON: [],
  geoJSON_style: '',
  KML: [],
  heatmap_options: '',
  
  init_map: function( id, x, y, zoom, type, is_route ) {
    this.id = id, 
    this.x = x, 
    this.y = y, 
    this.zoom = zoom, 
    this.type = type, 
    this.is_route = is_route,
    this.points = [],
    this.markers = [],
    this.route_color = [],
    this.markers = [],
    this.is_route = is_route,
    this.map_type = 0;
    this.bounds = null
    this.noXY = null, 
    this.noZoom = null;
    this.heatmap_locations = [];
    this.heatmap_points = [];
    this.heatmap_options = '';
    this.geoJSON = [];
    this.geoJSON_style = null;
    this.KML = [];
    
  },
  loadGeoJSON: function(j, style) {
    this.geoJSON.push(j);
    if (style) {
      try {
        this.geoJSON_style = JSON.parse(style);
      } catch (e) {
        alert($style+' is not a valid Google Data Layer style')
      }
    }
  },
  loadKML: function($j) {
    this.KML.push($j);
  },
  new_marker: function( lat, lon, desc, url, new_window, noXY, noZoom, iconFile, shadowFile, markerLabel )  {
    this.markers.push(
      {
        lat: lat,
        lon: lon,
        desc: desc,
        url: url,
        new_window: new_window,
        noXY: noXY,
        noZoom: noZoom,
        iconFile: iconFile,
        markerLabel: markerLabel
      }
    );
  },
  new_marker_extra: function( lat, lon, desc, url, new_window, noXY, noZoom, iconFile, shadowFile, markerLabel )
  {
    this.markers_extra.push(
      {
        lat: lat,
        lon: lon,
        desc: desc,
        url: url,
        new_window: new_window,
        noXY: noXY,
        noZoom: noZoom,
        iconFile: iconFile,
        markerLabel: markerLabel
      }
    );
  },
  polyline: function(color, width)
  {
    this.polyline_options = {color:color, width: width};
  },
  
  route: function(mode, suppressMarker) {
    this.route = {mode:mode, suppressMarker: suppressMarker};
  },
  set_bounds: function(noXY, noZoom, zoom) 
  {
    this.noXY = noXY, 
    this.noZoom = noZoom, 
    this.zoom = zoom;
  },
  fix_zoom: function(zoom) {
    this.zoom = zoom;
  },
  
  heatmap_options_add: function( options ) {
    try {
      this.heatmap_options = JSON.parse(options);  
    } catch (e) {
      // No valid option
    }
    
  },
  heatmap_point_add: function(lat, lng, weight)
  {
    if (weight !== null) {
      this.heatmap_locations.push({lat: lat, lng: lng, weight: weight});
    }
    this.heatmap_points.push({lat: lat, lng: lng});
  },
  heatmap_show: function()
  {
    
    // this.heatmap_options 
    var heatmap_locations = [];
    var heatmap_points = [], loc, po;
    for (var i = 0; i < this.heatmap_locations.length; i++) {
      loc = this.heatmap_locations[i];
      
      heatmap_locations.push( {location: new google.maps.LatLng(loc.lat, loc.lng), weight: loc.weight } );
    }
    for (i = 0; i < this.heatmap_points.length; i++) {
      po = this.heatmap_points[i];
      heatmap_points.push( new google.maps.LatLng(po.lat, po.lng) );
    }
    
    var heatmap = new google.maps.visualization.HeatmapLayer({
      data: (heatmap_locations.length ? heatmap_locations : heatmap_points),
      ...this.heatmap_options
    });
      
      heatmap.setMap(this.map);
      this.set_bounds_show(heatmap_points);
  },
  polyline_show: function() {
    var lat, lng, coords = [], poly;
  
    for(var i=0; i<this.points.length; i++) {
      lat = this.points[i].lat();
      lng = this.points[i].lng();
    
      coords.push({"lat":lat,"lng":lng});
    }
    poly = new google.maps.Polyline({
      path: coords,
      geodesic: true,
      strokeColor: this.polyline_options.color,
      strokeWeight: this.polyline_options.weight
    });
    poly.setMap(this.map);
  },
  route_show: function() {
    var p_from, p_to, waypoints = [], origin, destination;
    mode = this.route.mode.toUpperCase();
    if (!['DRIVING','WALKING','BICYCLING','TRANSIT'].includes(mode)) {
      window.alert('TravelMode should be driving, walking, bicycling or transit, not ' + mode);
    }
    const distinct = (value, index, self) => {
      return self.indexOf(value) === index;
    }
    var routes = [];
    var dcolors = this.route_color.filter( distinct);
    for (var i = 0; i < dcolors.length; i++) {
      routes.push({color: dcolors[i], point_ids: []});
    }
    for (var i = 0; i < this.route_color.length; i++) {
      routes[dcolors.indexOf(this.route_color[i])].point_ids.push(i);
    }
    
    
    
    for (var rind = 0; rind < routes.length; rind++) {
      waypoints = [];
      var cpoints = routes[rind].point_ids;
      var color = routes[rind]['color'];
      origin = {
        "lat":this.points[cpoints[0]].lat(),
        "lng":this.points[cpoints[0]].lng()
      };
      destination = {
        "lat":this.points[cpoints[cpoints.length-1]].lat(),
        "lng":this.points[cpoints[cpoints.length-1]].lng()
      };
      for(var p=1; p<cpoints.length-1; p++) {
        waypoints.push(
          { 
            location: {
              "lat":this.points[cpoints[p]].lat(),
              "lng":this.points[cpoints[p]].lng()
            }, 
            stopover: true
          }
        );
      }
      this.route_c(origin, destination, waypoints, mode, color, this.route.suppressMarker);
    }
    this.is_route = false;
    
  },
  route_c: function(origin, destination, waypoints, mode, color, suppressMarker)
  {
    var directionsService = new google.maps.DirectionsService();
    
    var themap = this.map;
    directionsService.route({
      origin: origin,
      destination: destination,
      waypoints: waypoints,
      travelMode: google.maps.TravelMode[mode]
    }, function (response, status) {
      if (status == google.maps.DirectionsStatus.OK) {
        var directionsDisplay = new google.maps.DirectionsRenderer({ draggable: true, map: themap,  polylineOptions: { strokeColor: color}, suppressMarkers: suppressMarker  });
        directionsDisplay.setDirections(response);
      } else {
        if (status=='ZERO_RESULTS') {
          window.alert('Unable to define a '+mode.toLowerCase()+" route between points");
        } else {
          window.alert('Directions request failed due to ' + status);
        }
      }
    });
    
  },
  
  colorname_to_rgb: function(name, glyph)
  {
    switch (name.toLowerCase()) {
    case 'blue':
      if (glyph) return '#0000FF';
      return '#6993FD';
      break;
    case 'green':
      if (glyph) return '#008000';
      return '#00E74C'
      break;
    case 'orange':
      if (glyph) return '#AA6E00';
      return '#FF9900'
      break;
    case 'purple':
      if (glyph) return '#5D4086';
      return '#9067FD'
      break;
    case 'yellow':
      if (glyph) return '#CDB77B';
      return '#FDF569'
      break;
    case 'red':
      if (glyph) return '#B31412';
      return '#FF0000';
      break;
    default:
      return name;
    }
  },
  
  /*
    Place a new marker to map. 
    noXY and noZoom will determine if the center and the zoom is done automatically or if the user set them
  
    iconFile can contain preinstalled icons (see markers and pushpins) or any image file reference with size/anchor position added to end (http://mypics.com/image.png:32:32::center)
  */
  
  
  markers_show: function(marker_ind) {
    var marker, newicon, markerInd, pushpinInd, iconSize;
    var markerOptions, image, icon_x, icon_y, icon_pos;
  
    // Inlcuded markers & pushpins
    var markers = [ 'blue', 'green', 'orange', 'pink', 'purple', 'red', 'yellow', 'blue-dot', 'green-dot', 'orange-dot', 'pink-dot', 'purple-dot', 'red-dot', 'yellow-dot' ];
    var pushpins = [ 'blue-pushpin', 'grn-pushpin', 'pink-pushpin', 'purple-pushpin', 'red-pushpin', 'ylw-pushpin' ];
    var new_marker = this.markers[marker_ind];
    var point = new google.maps.LatLng(new_marker.lat, new_marker.lon);
    var content;
    this.points.push(point);
    // this.points.push(point);
    if (this.is_route) {
      this.route_color.push(new_marker.desc);
      return;
    }
    var glyphColor, bg;
    var pin_options;
    
    if (new_marker.iconFile.startsWith('svg-marker-')) {
      new_marker.iconFile = 'marker'+new_marker.iconFile.substr(10);
    }
    if (markers.includes(new_marker.iconFile)) {
      if (new_marker.iconFile.substr(-3)=='dot') {
        var colorname = new_marker.iconFile.split("-")[0];
        borderColor = this.colorname_to_rgb(colorname, 1);
        bg = this.colorname_to_rgb(colorname);
        glyphColor = borderColor;
      } else {
        bg = this.colorname_to_rgb(new_marker.iconFile);
        borderColor = this.colorname_to_rgb(new_marker.iconFile, 1);
        glyphColor = bg;
      }
    } else if (new_marker.iconFile.startsWith('<svg')) {
      const parser = new DOMParser();
      content = parser.parseFromString( new_marker.iconFile, "image/svg+xml").documentElement;
    } else if (new_marker.iconFile.startsWith('marker-')) {
      var cp = new_marker.iconFile.split('-');
      if (cp.length>1) {
        try {
          pin_options = JSON.parse(cp[1]);
        } catch (e) {
          bg = this.colorname_to_rgb(cp[1]);
          glyphColor = this.colorname_to_rgb(cp[1]);
          borderColor = this.colorname_to_rgb(cp[1]);
          if (cp.length>2) {
            glyphColor = this.colorname_to_rgb(cp[2]);
            borderColor = this.colorname_to_rgb(cp[2]);
          }
          if (cp.length>3) {
            borderColor = this.colorname_to_rgb(cp[3]);
          }
        }
      }
    } else if (new_marker.iconFile) {
      if (pushpins.includes(new_marker.iconFile)) {
        new_marker.iconFile = 'https://maps.google.com/mapfiles/ms/micons/'+new_marker.iconFile+'.png';
      }
      content = document.createElement("img");
      var src = new_marker.iconFile.split('::');
      if (src[1]) {
        content.width = src[1];
        if (src[2]) {
          content.height = src[1];
        }        
      }
      content.src = src[0];
    }
    if (content===undefined) {
      var pin;
      if (pin_options) {
        pin = new google.maps.marker.PinElement(pin_options);
      } else if (bg) {
        pin = new google.maps.marker.PinElement({
          background: bg,
          glyphColor: glyphColor,
          borderColor: borderColor
        });
      } 
      if (pin == undefined) {
        pin = new google.maps.marker.PinElement();
      }
      content = pin.element;
    }
    marker = new google.maps.marker.AdvancedMarkerElement({
      position: point,
      map: this.map,
      content: content
    });
    marker.url = new_marker.url;
    marker.new_window = new_marker.new_window;
    
    // new_marker.push(marker);
    
    var infowindow = new google.maps.InfoWindow();
    google.maps.event.addListener(marker, 'click', function() {
      if (marker.url) {
        if (new_marker.new_window) {
          window.open(marker.url);
        } else {
          document.location.href = marker.url;
        }
      } else {
        infowindow.setContent(new_marker.desc);
        infowindow.open(this.map,marker);
      }
    });
  },
  set_bounds_show: function(points)
  {
    var i, p;
    
    if (points !== undefined && points.length!=0) {
      this.points = points; 
    }
    if (this.noXY || this.noZoom ) {
      var bounds = new google.maps.LatLngBounds();

      for (i=0; i < this.points.length; i++) {
          var ll = new google.maps.LatLng(this.points[i].lat(), this.points[i].lng());
          bounds.extend(ll);
      }
    
      if (this.points.length==1 && this.map.getZoom()===0) {
        if (this.noXY) {
          this.map.setCenter(bounds.getCenter());
        }
        if (this.noZoom) {
          this.map.setZoom(7);
        }
      } else {
        this.map.fitBounds(bounds);
      }
    }
  },
  fix_zoom2: function() 
  {
    var map = this.map, mapzoom = this.zoom;
    var listener = google.maps.event.addListener(this.map, "idle", function() {
      if (map.getZoom() !== mapzoom && mapzoom!==undefined) map.setZoom(mapzoom);
        google.maps.event.removeListener(listener);
    });
  },
  get_map_object: function(resultclass)
  {
    var id = $('.'+resultclass).find('.mydbr_googlemap').attr('id');
    if (id) {
      return window['ext_gm_'+id];
    } else {
      return null;
    }
  } 
};