<?php

class ExtGeoLocation {
  private $language;
  private $service_provider;
  private $providers;
  public  $procedure;
  
  public function __construct($options)
  {
    global $mydbr_defaults;

    $this->providers = $mydbr_defaults['ip2location'];

    $lang_opt = 'dbr.ip2location.language';
    $this->language = isset($options[$lang_opt]) ? $options[$lang_opt]['lang'] : 'en';
    
    $lang_opt = 'dbr.ip2location.language';
    $this->language = isset($options[$lang_opt]) ? $options[$lang_opt]['lang'] : 'en';
  
    $cmd = 'dbr.ip2location';
    $this->service_provider = isset($options[$cmd]['service_provider']) ? $options[$cmd]['service_provider'] :
       $this->providers['default_service_provider'];
    
    $cmd = 'dbr.ip2location';
    $this->procedure = isset($options[$cmd]['procname']) ? $options[$cmd]['procname'] : 'null';
  }

  private function get_url($url, $username = null, $password = null)
  {
    $ch = curl_init(); 
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt( $ch, CURLOPT_RETURNTRANSFER,  true );
    curl_setopt($ch, CURLOPT_HTTPHEADER, array('Accept: application/json'));
    if ($username && $password) {
      curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC); 
      curl_setopt($ch, CURLOPT_USERPWD, $username.':'.$password);
    }
    $response = @curl_exec($ch);
    if ($response===false) {
      error_print(curl_error($ch), 'center','comment');
      return null;
    }
    curl_close($ch); 
    
    return $response;
  }
  
  private function location_ip2location($ip)
  {
    $ip2 = $this->providers['ip2location'];
    
    $url = 'https://api.ip2location.com/?ip='.$ip.'&key='.$ip2['api_key'].'&package='.$ip2['package'].'&format=json';
    $data = json_decode((string) $this->get_url($url));
    if (isset($data->response) && $data->response!="OK") {
      error_print($ip.': '.$data->response, 'center','comment');
      $data = null;
    } else {
      $data = array(
        'ip' => $ip,
        'country' => isset($data->country_name) ? $data->country_name : '',
        'city' => isset($data->city_name) ? $data->city_name : '',
        'latitude' => $data->latitude,
        'longitude' => $data->longitude,
        'isp' => isset($data->isp) ? $data->isp : ''
      );
    }
    return $data;
  }
  
  private function location_ipinfo($ip)
  {
    $ipinfo = $this->providers['ipinfo'];
    
    $url = 'https://ipinfo.io/json?ip='.$ip.'&token='.$ipinfo['token'];
    
    $data = json_decode((string) $this->get_url($url));
    
    if (isset($data->error)) {
      error_print($ip.': '.$data->error['message'], 'center','comment');
      $data = null;
    } else {
      $loc = explode(',', $data->loc);
      $data = array(
        'ip' => $ip,
        'country' => isset($data->country) ? $data->country : '',
        'city' => isset($data->city) ? $data->city : '',
        'latitude' => $loc[0],
        'longitude' => $loc[1],
        'isp' => ''
      );
    }
    return $data;
  }
  
  private function location_maxmind($ip)
  {
    $url = 'https://geoip.maxmind.com/geoip/v2.1/city/'.$ip;
    $json_data = $this->get_url($url, $this->providers['maxmind']['account_id'], $this->providers['maxmind']['license_key']);
    $data = json_decode((string) $json_data);

    if (isset($data->error)) {
      error_print($data->error, 'center','comment');
      $data = null;
    } else {
      $l = $this->language;
      $data = array(
        'ip' => $data->traits->ip_address,
        'country' => isset($data->country->names->$l) ? $data->country->names->$l : '',
        'city' => isset($data->city->names->$l) ? $data->city->names->$l : '',
        'latitude' => $data->location->latitude,
        'longitude' => $data->location->longitude,
        'isp' => isset($data->traits->isp) ? $data->traits->isp : ''
      );
    }
    return $data;
  }
  
  public function location($ip)
  {
    switch ($this->service_provider) {
      case 'maxmind':
        $data = $this->location_maxmind($ip);
        break;
      case 'ip2location':
        $data = $this->location_ip2location($ip);
        break;
      case 'ipinfo':
        $data = $this->location_ipinfo($ip);
        break;
    }
    return $data;
  }
}

function IP2Location_GetLocations( $id, $options, $dataIn, $colInfo )
{
  $ip2location = new ExtGeoLocation( $options );

  for ($i=0; $i < sizeof($dataIn); $i++) {
    $ip = !empty($dataIn[$i][0]) ? trim($dataIn[$i][0]) : '';
    $loc = $ip2location->location($ip);
    
    if ($loc) {
        $tSQLCmd = $ip2location->procedure.' '.
        SQL_String($loc['ip']) . ','.
        SQL_String($loc['country']) . ','.
        SQL_String($loc['city']) . ','.
        SQL_NumFloat($loc['latitude']) . ','.
        SQL_NumFloat($loc['longitude']) . ','.
        SQL_String($loc['isp']);
        
        SQL_Fetch($tSQLCmd);
    }
  }
}
