<?php
/*
    Copyright mydbr.com https://mydbr.com 2007-2024
    You are free to modify this file
*/

use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception;
use PHPMailer\PHPMailer\SMTP;
use PHPMailer\PHPMailer\OAuth;
use League\OAuth2\Client\Provider\Google;

require 'lib/external/vendor/autoload.php';


class myDBR_PHPMailer extends PHPMailer {
  public $errorcnt = 0;
  public $mail_errors = [];
  public $attachment_error_count = 0;
  public $errornous_attachments = '';
  public $postprocess_type = '';
  public $advanced_conversion = false;
  public $advanced_conversion_cache = [];
  
  public function remove_private_attachments( $filenames )
  {
    foreach ($filenames as $filename) {
      foreach ($this->attachment as $key => $attachment) {
        if ($filename == $attachment[1]) {
          unset($this->attachment[$key]);
        }
      }
    }
    $this->attachment = array_values($this->attachment);
  }
  public function postProcess($html)
  {
    if (isset($this->advanced_conversion_cache[$html])) {
      return $this->advanced_conversion_cache[$html];
    }
    $descriptorspec = array( 0 => array("pipe", "r"), 1 => array("pipe", "w") );
    $process = proc_open($notify_successful_mail = Extension::get_mail_setting('postprocess_cmd').' -f html -t '.$this->postprocess_type, $descriptorspec, $pipes);
    if (is_resource($process)) {
      fwrite($pipes[0], $html);
      fclose($pipes[0]);
      $result = stream_get_contents($pipes[1]);
      fclose($pipes[1]);
      $return_value = proc_close($process);
      
      if ($return_value) {
        $this->errorcnt++;
        $this->ErrorInfo = "Postprocess-call failed with return value ".$return_value;
      }
      $this->advanced_conversion_cache[$html] = $result;
      
      return $result;
    } else {
      $this->errorcnt++;
      $this->ErrorInfo = "proc_open for mail conversion failed.";

      
      return is_null($html) ? '' : html_entity_decode(trim(strip_tags(preg_replace('/<(head|title|style|script)[^>]*>.*?<\/\\1>/si', '', (string) $html))),ENT_QUOTES,$this->CharSet);
    }
  }
}

function Ext_Mail_AddrType($type)
{
  if (is_null($type)) $type='';
  switch (strtolower($type)) {
    case 'cc':
      return 'addCC';
    case 'bcc':
      return 'addBCC';
    default:
      return 'addAddress';
  }
}

/*
    This is the function described in extensions.php
    $id: object id in the report
    $options: command options described in extensions.php cmds, contains the parsed values from the report
    $dataIn: Array of: recipient_email,recipient_name,subject,body + extra columns for dbr.report attachments
    $colInfo: column info - not needed in mail
*/

function Ext_Mail( $id, $options,  $dataIn, $colInfo )
{
  global $mydbr_mail;

  $mail_extension = new Extension;

  $attach_error = '';
  ob_start();
  $mail = new myDBR_PHPMailer();
  $mail->CharSet = 'UTF-8';

  $mail->IsSMTP(); // telling the class to use SMTP

  $mailer = Extension::get_mail_setting('mailer');
  
  // Is the email address defined in the report?
  if (isset($options['dbr.mail.sender']['email'])) {
    $email = $options['dbr.mail.sender']['email'];
    $name = isset($options['dbr.mail.sender']['name']) ? $options['dbr.mail.sender']['name'] : '';
    $replyto_email = isset($options['dbr.mail.sender']['replyto_email']) ? $options['dbr.mail.sender']['replyto_email'] : $email;
    $replyto_name = isset($options['dbr.mail.sender']['replyto_name']) ? $options['dbr.mail.sender']['replyto_name'] : $name;
  } else {
    $email = Extension::get_mail_setting('from_email');
    $name = Extension::get_mail_setting('from_name');
    $replyto_email = Extension::get_mail_setting('replyto_email');
    $replyto_name = Extension::get_mail_setting('replyto_name');
  }
  switch ($mailer) {
    case 'mail':
      $mail->isMail();
      break;
    case 'sendmail':
      $mail->isSendmail();
      break;
    case 'qmail':
      $mail->isQmail();
      break;
    default:
    
      if (isset($options['dbr.mail.smtp.server']['host'])) {
        $mail->Host = $options['dbr.mail.smtp.server']['host'];
        $mail->Port = $options['dbr.mail.smtp.server']['port'];
        $mail->SMTPSecure = isset($options['dbr.mail.smtp.server']['secure']) ? $options['dbr.mail.smtp.server']['secure'] : '';
      } else {
        $mail->Host = Extension::get_mail_setting('host');
        $mail->Port = Extension::get_mail_setting('port');
        $mail->SMTPSecure = Extension::get_mail_setting('secure');
      }
      $smtp_auth = Extension::get_mail_setting('smtp_auth');
      $clientId = Extension::get_mail_setting('clientid');
      $clientSecret = Extension::get_mail_setting('clientsecret');
      $refreshToken = Extension::get_mail_setting('refreshtoken');
      
      if (isset($options['dbr.mail.smtp.autotls']['autotls'])) {
        $mail->SMTPAutoTLS = $options['dbr.mail.smtp.autotls']['autotls'] ? true : false;
        if (!$mail->SMTPAutoTLS) {
          $mail->SMTPSecure = false;
        }
      }
      if (Extension::get_mail_smtp_ipv4()) {
        // If IPV6 SMTP is blocked, use: $mydbr_defaults['mail']['force_ipv4'] = true;
        if ($mail->SMTPSecure) {
          // Now that we will be using IP instead of name, we'll use the given host as SSL peer_name option
          $mail->SMTPOptions = array( 'ssl' => array( 'peer_name' => $mail->Host ) );
        }
        // Convert the hostname to IPv4 
        $mail->Host = gethostbyname($mail->Host);
      }
      $mail->SMTPKeepAlive = true; // SMTP connection will not close after each email sent
      $mail->SMTPDebug = isset($options['dbr.mail.debug']['debug']) ? $options['dbr.mail.debug']['debug'] : Extension::get_mail_setting('debug');

      $username = null;
      $password = null;

      if (isset($options['dbr.mail.smtp.ssl'])) {
        for ($i=0; $i < sizeof($options['dbr.mail.smtp.ssl']['option']); $i++) { 
          $value = $options['dbr.mail.smtp.ssl']['value'][$i];
          if ($value == 'false' || $value == 'true') {
            $value = ($value == 'true');
          }
          $mail->SMTPOptions['ssl'][$options['dbr.mail.smtp.ssl']['option'][$i]] = $value;
        }
      }
      if ($smtp_auth=='xoauth2') {
        $mail->SMTPAuth = true;
        $mail->AuthType = 'XOAUTH2';

        $provider = new Google(
          [
            'clientId' => $clientId,
            'clientSecret' => $clientSecret,
          ]
        );
        $mail->setOAuth(
          new OAuth(
            [
              'provider' => $provider,
              'clientId' => $clientId,
              'clientSecret' => $clientSecret,
              'refreshToken' => $refreshToken,
              'userName' => $email,
            ]
          )
        );
      } else {
        if (isset($options['dbr.mail.smtp.user']['user']) && isset($options['dbr.mail.smtp.user']['password'])) {
          $username = $options['dbr.mail.smtp.user']['user'];
          $password = $options['dbr.mail.smtp.user']['password'];
        } else {
          $username = Extension::get_mail_setting('username');
          $password = Extension::get_mail_setting('password');
        }
        // Enable SMTP authentication?
        if ($username && $password) {
          $mail->Username = $username;
          $mail->Password = $password;
          $mail->SMTPAuth = true;
        }
        // Additional options
        if (isset($options['dbr.mail.smtp.authentication.type']['type']) &&
          in_array($options['dbr.mail.smtp.authentication.type']['type'], array('LOGIN', 'PLAIN', 'NTLM', 'CRAM-MD5', 'XOAUTH2')) ) {

          $mail->AuthType = $options['dbr.mail.smtp.authentication.type']['type'];
        }
        
      }
      break;
  }
  $notify_successful_mail = Extension::get_mail_setting('notify_successful_mail');
  if (isset($options['dbr.mail.notify_successful_mail']['noyes'])) {
    $notify_successful_mail = $options['dbr.mail.notify_successful_mail']['noyes'];
  }
  if (isset($options['dbr.mail.encoding']['encoding'])) {
    $mail->Encoding = $options['dbr.mail.encoding']['encoding'];
  }

  $mail->AllowEmpty = true;
  $mail->SetFrom($email, $name);
  if ($replyto_email) {
    $mail->AddReplyTo($replyto_email, $replyto_name);
  }
  if (isset($options['dbr.mail.attach']) && isset($options['dbr.mail.attach']['report'])) {
    $ignore_ssl = isset($options['dbr.mail.attach']['ignore_ssl']) ? $options['dbr.mail.attach']['ignore_ssl'] : false;
    if (!is_array($options['dbr.mail.attach']['report'])) {
      $options['dbr.mail.attach']['report'] = [$options['dbr.mail.attach']['report']];
      $options['dbr.mail.attach']['name'] = [$options['dbr.mail.attach']['name']];
    }
    for ($i=0; $i < sizeof($options['dbr.mail.attach']['report']); $i++) {
      if (isset($options['dbr.mail.attach']['ignore_ssl'])) {
        $ignore_ssl = $options['dbr.mail.attach']['ignore_ssl'];
      }
      $http_code = $mail_extension->mail_add_report_attachments($mail, $options['dbr.mail.attach']['name'][$i], $options['dbr.mail.attach']['report'][$i], $ignore_ssl, $attach_error );
      if ($http_code!==200 || $attach_error) {
        $mail->attachment_error_count += 1;
        $mail->errornous_attachments .= "\n" .$options['dbr.mail.attach']['name'][$i]. ': HTTP Code: '.$http_code.($attach_error ? ', Error: '.$attach_error : '');
      }
    }
  }
  if (isset($options['dbr.mail.postprocess']['format'])) {
    $mail->postprocess_type = $options['dbr.mail.postprocess']['format'];
    $mail->advanced_conversion = array($mail, 'postProcess');
  }

  /*
    Only if there is one recipient in mail
  */
  $mail_to = '';
  for ($i=0; $i<sizeof($dataIn) ; $i++) {    // send the mail one by one
    $mail->ErrorInfo = '';
    $mail_to = '';
    // Will be generated per recipient
    $private_attachments = array();
    $error_in_attachment_name = array();
    
    // Avoiding to overload the server
    // set_time_limit(30); // sets (or resets) maximum  execution time to 30 seconds

    // Do not send mails too fast (mail server might refuse to send the mails)
    if ( ($i % Extension::get_mail_setting('sleep_after_rows')) == 0) {
      usleep(Extension::get_mail_setting('sleep_microsec'));
    }
    $mail->Subject = $dataIn[$i][2];

    $urls  = Extension::get_report_links($dataIn[$i], null, true);
    foreach ($urls as $attachment) {
      $attach_error = 0;
      $attach_type = substr($attachment[1],-4);
      if (in_array($attach_type, array('=xls', 'xlsx', '=pdf', '=sql', '=csv'))) {
        $file_ext = '.'.str_replace('=', '', $attach_type);
        if (!_str_ends_with($attachment[0], $attach_type)) {
          $attachment[0] .= $file_ext;
        }
        $http_code = $mail_extension->mail_add_report_attachments($mail, $attachment[0], $attachment[1], false, $attach_error );
        if ($http_code!=200 || $attach_error) {
          $mail->attachment_error_count += 1;
          $mail->ErrorInfo = 'An attachment error';
          $mail->errornous_attachments .= "\n" .$attachment[0]. ': HTTP Code: '.$http_code.($attach_error ? ', Error: '.$attach_error : '');
        }
        $private_attachments[] = $attachment[0];
      } else {
        if (!in_array($attachment[0], $error_in_attachment_name)) {
          $error_in_attachment_name[] = $attachment[0];
          $mail->ErrorInfo = 'An attachment error';
          $mail->errornous_attachments .= "\n" .('(Name missing:'.$attachment[0].')');
        }
      }
    }
    if (isset($options['dbr.mail.recipient'])) {
      for ($recip=0; $recip < sizeof($options['dbr.mail.recipient']['email']); $recip++) {
        $recipient = $options['dbr.mail.recipient'];
        $cc_bcc = isset($recipient['cc_bcc'][$recip]) ? $recipient['cc_bcc'][$recip] : '';
        if (!$cc_bcc) {
          $mail_to .= ($mail_to ? ',' : '') . $recipient['email'][$recip];
        }
        $addr_type = Ext_Mail_AddrType($cc_bcc);
        
        $email = $options['dbr.mail.recipient']['email'][$recip];
        if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
          if (isset($options['dbr.mail.ignore_invalid_email']['ignore']) && $options['dbr.mail.ignore_invalid_email']['ignore']) {
            $mail->mail_errors[] = ['to' => $email, 'error' => 'Recipient ignored. Invalid email address', 'attachments' => ''];
            continue;
          }
        }
        $mail->$addr_type($email, isset($recipient['name'][$recip]) ? $recipient['name'][$recip] : '');
      }
    }
    if (sizeof($dataIn)==1 && isset($dataIn[$i][0]) && $dataIn[$i][0]) {
      $mail_to .= ($mail_to ? ',' : '').$dataIn[$i][0];
    }
    
    if (strpos($dataIn[$i][0], ',')!==false) {
      $addresses = explode(',', (string) $dataIn[$i][0]);
      foreach ($addresses as $address) {
        $mail->addAddress($address, '');
        $mail_to .= ($mail_to ? ',' : '').$address;
      }
    } elseif ($dataIn[$i][0]) {
      $mail->addAddress($dataIn[$i][0], $dataIn[$i][1]);
      $mail_to .= ($mail_to ? ',' : '').$dataIn[$i][0];
    }
    
    $body = (isset($dataIn[$i][3]) ? $dataIn[$i][3] : '');

    $record = $mail_extension->recording_get();
    if (isset($options['dbr.mail']['HTML']) && $options['dbr.mail']['HTML']==1) {
      /* Get HTML content */
      $mail->AltBody = '';
      if ($record) {
        /* Attach images as attachments */
        $record = $mail_extension->mail_add_attachments( $record, $mail );
        $replace_from = Extension::get_mail_setting('replace_from');
        $replace_to = Extension::get_mail_setting('replace_to');
        /* Outlook uses Word HTML parser which does not support multiple css classes -> fixing it */
        $replace_from = array(
          'class="cell align_l',
          'class="cell align_r',
          'class="cell_header align_l',
          'class="cell_header align_r'
        );
        $replace_to = array(
          'class="cell_left align_l',
          'class="cell_right align_r',
          'class="cell_header_left align_l',
          'class="cell_header_right align_r'
        );
        $record = is_null($record) ? null : str_replace($replace_from, $replace_to, $record);

        /* 
          Apply CSS styles to mail elements 
          Only allow from user-directory 
        */
        if (isset($options['dbr.mail.header_file']['filename']) && 
            strpos($options['dbr.mail.header_file']['filename'], '..')===false && 
            file_exists('user/'.$options['dbr.mail.header_file']['filename'])) {
          $hdr_file = 'user/'.$options['dbr.mail.header_file']['filename'];
        } else {
          $hdr_file = 'extensions/mail/mail_header.html';
        }
        $hdr = file_get_contents($hdr_file);
        $user_mail_css = 'user/mail.css';
        if (file_exists($user_mail_css)) {
          $hdr .= file_get_contents($user_mail_css);
        }
        $hdr .= chr(10).'</style>'.chr(10).'</head>'.chr(10).'<body>';
        $mail->MsgHTML($hdr.$record.'</body></html>', '', $mail->advanced_conversion);
      } else {
        if (isset($options['dbr.mail.nobr.html']['value']) && $options['dbr.mail.nobr.html']['value']==1) {
          $mail->MsgHTML($body, '', $mail->advanced_conversion);
        } else {
          $mail->MsgHTML(nl2br($body), '', $mail->advanced_conversion);
        }
      }
    } else {
      $body = $record.$body;
      $mail->Body = is_bool($mail->advanced_conversion) ? $body : call_user_func($mail->advanced_conversion, $body);
      $mail->IsHTML(false);
    }
    if ($mail->ErrorInfo || !$mail->send()) {
      $mail->errorcnt++;
      $mail->mail_errors[] = ['to' => $mail_to, 'error' => $mail->ErrorInfo, 'attachments' => $mail->errornous_attachments];
    }
    
    if (isset($options['dbr.mail.log.proc']['procedure'])) {
      $optional_params = [];
      for ($params=1; $params < 11; $params++) { 
        if (isset($options['dbr.mail.log.proc']['p'.$params])) {
          $optional_params[] = $options['dbr.mail.log.proc']['p'.$params];
        } else {
          break;
        }
      }
      
      $mail_extension->mail_log( 
        $options['dbr.mail.log.proc']['procedure'], 
        $mail->ErrorInfo ? 'error' : 'ok', 
        $mail->ErrorInfo,
        $mail_to,
        $mail->errornous_attachments,
        $dataIn[$i],
        $colInfo,
        $optional_params
      );
    }
    
    $mail->remove_private_attachments($private_attachments);
    $mail->ClearAddresses();
  }
  $mail_extension->recording_clear();
  $mail->SmtpClose();
  $mail_extension->mail_clear_attachments();

  $captured = ob_get_contents();
  ob_end_clean();
  if ($mail->SMTPDebug>0) {
    echo '<div class="box debug_code"><b>Debug</b><br><br>'.
      ($captured ? $captured : $mail->ErrorInfo).
      '</div>';
  }
  if ($attach_error) {
    $mail->ErrorInfo .= ' Attachment error: '.$attach_error;
  }
  
  if ($notify_successful_mail) {
    $msg = 'Sent '.sizeof($dataIn).' mail'.(sizeof($dataIn)>1 ? 's' : '')
      .' with ' . ($mail->errorcnt ? $mail->errorcnt.' error'. ($mail->errorcnt>1 ? 's' : '') : 'no errors')
      .($mail->attachment_error_count ? '. Errors in '.$mail->attachment_error_count.' attachment'.($mail->attachment_error_count>1 ? 's' : '') : '' );
      
      if ($mail->errornous_attachments) {
        $msg .= ' '.$mail->errornous_attachments;
      }
      
      echo '<div class="comment prediv">'. $msg .'</div>';

      if (sizeof($mail->mail_errors)>0) {
        ?>
        <table class="rstable">
          <thead>
            <tr class="row_header">
              <th class="cell_header">To</th>
              <th class="cell_header">Error</th>
              <th class="cell_header">Attachment error</th>
            </tr>
          </thead>
          <tbody>
<?php 
        foreach ($mail->mail_errors as $error) {
          ?>
          <tr class="row">'
            <td class="cell"><?php echo htmlentities($error['to'], ENT_COMPAT, 'UTF-8'); ?></td>
            <td class="cell"><?php echo htmlentities($error['error'], ENT_COMPAT, 'UTF-8'); ?></td>
            <td class="cell"><?php echo htmlentities($error['attachments'], ENT_COMPAT, 'UTF-8'); ?></td>
          </tr>
<?php
        }
        echo '</tbody></table>';
      }
  }
}
