/*
  Copyright mydbr.com https://mydbr.com 2007-2024
  if you need to modify this file, make a copy of it into the user/extensions as it will be overridden durung the updates
*/

var OSM_map = {
  map: null,
  mapId : '',
  points : [],
  markers : [],
  heatmap : [],
  heatmapLatLng : [],
  geoJsonBounds : [],
  polygonData : [],
  polygons : [],
  tilesUrl : null,
  zoom: null,
  isHeatmap: 0,
  heatmapOptions: '',
  tiles: null,
  tilesOptions: {},
  isShown: false,

  initMap: function( id, mapOptions, zoom, isHeatmap, heatmapOptions ) {
    this.points = [];
    this.markers = [];
    this.heatmap = [];
    this.heatmapLatLng = [];
    this.geoJsonBounds = [];
    this.polygons = [];
    this.polygonData = [];
    this.tilesUrl = null;
    this.mapId = 'map'+id;
    this.zoom = zoom;
    this.map = L.map(this.mapId, mapOptions);
    this.isHeatmap = isHeatmap;
    this.heatmapOptions = Object.keys(heatmapOptions).length === 0 && heatmapOptions.constructor === Object ? {radius: 25} : heatmapOptions;
    this.tiles = null;
    this.tilesOptions = {};
    this.isShown = false;
    return this.map;
  },
  newPolygon: function( latlngs, options)
  {
    this.polygonData.push(
      {latlngs: latlngs, options: options}
    );
  },
  setTiles: function( tiles, options )
  {
    this.tiles = tiles;
    this.tilesOptions = options;
  },
  newMarker: function( lat, lng, title, heatmapWeight, markerOptions )
  {
    var marker;
    if (this.isHeatmap) {
      this.heatmap.push([lat, lng, heatmapWeight ? heatmapWeight : 1]);
      this.heatmapLatLng.push(L.latLng(lat, lng));
    } else {
      if (Object.keys(markerOptions).length === 0 && markerOptions.constructor === Object) {
        marker = L.marker([lat, lng, 1]).addTo(this.map);
      } else {
        if (markerOptions.iconUrl === undefined) {
          marker = L.marker([lat, lng, 1], {icon: L.ExtraMarkers.icon(markerOptions)}).addTo(this.map);
        } else {
          marker = L.marker([lat, lng, 1], {icon: L.icon(markerOptions)}).addTo(this.map);
        }
      }
      this.markers.push(marker);
      if (title !== undefined) {
        marker.bindPopup(title);
      }
    }
  },
  show: function() {
    var polygon, bounds = null, geob, group, tooltip, options, i;
    
    this.isShown = true;
    if (!this.tiles) {
      this.tiles = 'https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png';
    }

    if (!this.tilesOptions.attribution) {
      this.tilesOptions.attribution = '<a href="myDBR.com">myDBR</a> | Map data &copy; <a href="https://www.openstreetmap.org/">OpenStreetMap</a> contributors, <a href="https://creativecommons.org/licenses/by-sa/2.0/">CC-BY-SA</a>';
    }
    L.tileLayer(this.tiles, this.tilesOptions).addTo(this.map);
    for (i = 0; i < this.polygonData.length; i++) {
      tooltip = null;
      options = this.polygonData[i].options;
      if ('tooltip' in options) {
        tooltip = Object.assign({}, options.tooltip);
        delete options.tooltip;
      }
      polygon = L.polygon(this.polygonData[i].latlngs, options).addTo(this.map);
      if (tooltip) {
        polygon.bindTooltip(tooltip.title, tooltip.options);
      }
      this.polygons.push(polygon);
      if (bounds === null) {
        bounds = polygon.getBounds();
      } else {
        bounds.extend(polygon.getBounds());
      }
    }
    if (this.markers.length>0) {
      group = new L.featureGroup(this.markers);
      if (bounds === null) {
        bounds = group.getBounds();
      } else {
        bounds.extend(group.getBounds());
      }
      for (i = 0; i < this.geoJsonBounds.length; i++) {
        geob = new L.LatLngBounds(this.geoJsonBounds[i].getSouthWest(), this.geoJsonBounds[i].getNorthEast());
        bounds.extend(geob);
      }
      this.map.fitBounds(bounds);
    } else {
      if (this.isHeatmap) {
        var heat = L.heatLayer(this.heatmap, this.heatmapOptions).addTo(this.map);
        if (bounds === null) {
          bounds = L.latLngBounds( this.heatmapLatLng);
        } else {
          bounds.extend(L.latLngBounds( this.heatmapLatLng));
        }
        this.map.fitBounds(bounds);
      } else {
        if (bounds === null) {
          marker = L.marker([60.17247,24.9346133]).addTo(this.map);
          this.markers.push(marker);
          group = new L.featureGroup(this.markers);
          bounds = group.getBounds();
        }
        this.map.fitBounds(bounds);
      }
    }
    if (this.zoom) {
      this.map.setZoom(this.zoom);
    }
    return bounds;
  },
  geoJSON : function(gj)
  {
    var l = L.geoJSON(gj).addTo(this.map);
    this.geoJsonBounds.push(l.getBounds());
  },
  get_map_object: function(resultclass)
  {
    var id = $('.'+resultclass).find('.mydbr_osm').attr('id');
    if (id) {
      return window['ext_osm_'+id];
    } else {
      return null;
    }
  },
  showMaps: function()
  {
    var b;
    for (var i=0; i < OSM_maps.length; i++) {
      b = null;
      OSM_map = OSM_maps[i].map;
      if (!OSM_map.is_shown) b = OSM_map.show();
        if (OSM_maps[i].gjs_bounds !== null) { 
          if (OSM_maps[i].bounds == null && b !== null) {
            OSM_maps[i].bounds = b;
          }
          OSM_maps[i].mapobj.fitBounds(OSM_maps[i].bounds); 
        }
    } 
    OSM_maps = [];
  }
};