<?php 


class OpenStreetMap {
  public $geocode_server = '';
  public $geocode_server_osm_node = '';
  
  function __construct()
  {
    global $mydbr_defaults;
    
    $this->geocode_server = $mydbr_defaults['openstreetmap']['geocode'];
    $this->geocode_server_osm_node = $mydbr_defaults['openstreetmap']['osm_node_geocode'];
  }
  
  function get_coordinates( $address, $mode = 'address' )
  {
    $lat_long = Extension::geocode($address);
    if (is_array($lat_long) && sizeof($lat_long)==2) {
      return $lat_long;
    }
  
    if ($mode=='address') {
      $url = $this->geocode_server.urlencode((string) $address);
    } else {
      $url = $this->geocode_server_osm_node.urlencode((string) $address);
    }
    $json = get_url_content($url, $error, 20, true);
    
    $data = json_decode((string) $json);

    $lat_long = array();

    if ($data && sizeof($data)>0) {
      $ll = $data[0];
      if (property_exists($ll, 'lat') && property_exists($ll, 'lon')) {
        $lat_long = array($ll->lat, $ll->lon);
      }
    }
    
    if (sizeof($lat_long)==2) {
      Extension::geocode_set($address, $lat_long);
      return $lat_long;
    }
    
    return null;
  }
}

function Ext_OpenStreetMap( $id, $options,  $dataIn, $colInfo )
{
  // Default size of the map
  $defWidth = 800;
  $defHeight = 600;

  $osm = new OpenStreetMap();
  $osm_options = $options['dbr.openstreetmap'];

  $map_options = '{}';
  if (isset($options['dbr.openstreetmap.options']['options'])) {
    $map_options = $options['dbr.openstreetmap.options']['options'];
  }

  $heatmap = 0;
  $heatmap_options = '{}';
  if (isset($options['dbr.openstreetmap.heatmap']['enabled'])) {
    $heatmap = 1;
    if (isset($options['dbr.openstreetmap.heatmap']['options'])) {
      $heatmap_options = $options['dbr.openstreetmap.heatmap']['options'];
    }
  }

  $mode = isset($osm_options['mode']) ? $osm_options['mode'] : '';

  if (!in_array($mode, array('coordinates', 'address', 'coordinates_address', 'osm_node'))) {
    error_print("Usage: select 'dbr.openstreetmap', 'coordinates' | 'address' | 'coordinates_address' | osm_node {,title, width, height, zoom}");
    return;
  }

  // Does the report define map dimensions?
  $width = isset($osm_options['width']) ? $osm_options['width'] : $defWidth;
  $height = isset($osm_options['height']) ? $osm_options['height'] : $defHeight;
  if (is_numeric(substr($width,-1))) {
    $width .= 'px';
  }
  if (is_numeric(substr($height,-1))) {
    $height .= 'px';
  }
  $zoom = isset($osm_options['zoom']) ? intval($osm_options['zoom']) : 'null';

  $tiles = isset($options['dbr.openstreetmap.tiles']['url']) ? $options['dbr.openstreetmap.tiles']['url'] : null;
  $tiles_options = isset($options['dbr.openstreetmap.tiles']['options']) ? $options['dbr.openstreetmap.tiles']['options'] : '{}';
  if (substr($tiles_options, 0, 1)!=='{' && substr($tiles_options, -1) !=='}') {
    $tiles_options = '{'.$tiles_options.'}';
  }
  $geojson = isset($options['dbr.openstreetmap.geojson']['url']) ? $options['dbr.openstreetmap.geojson']['url'] : null;
  if ($geojson && !is_array($geojson)) {
    $geojson = array($geojson);
  }
  $geojson_options = isset($options['dbr.openstreetmap.geojson']['options']) ? $options['dbr.openstreetmap.geojson']['options'] : null;
  if ($geojson_options && !is_array($geojson_options)) {
    $geojson_options = array($geojson_options);
  }
  $polygons = Extension::multi_convert(isset($options['dbr.openstreetmap.polygon']) ? $options['dbr.openstreetmap.polygon'] : null);
  
  // Placeholder for the map, executed JavaScript will replace the content of this with the map
  // Ext_KeepWithNext keeps the map at same line as the previous element if so wanted (dbr.keepwithnext)
  
  ?>
  <div <?php echo Extension::resultclass(true, 'map_wrapper', true); ?>>
  <?php
  // If the title attribute has been set we'll use subtitle class to draw it
  if (isset($osm_options['title'])) {
    echo '<div class="subtitle" style="text-align: center;padding-bottom:2px;">'. $osm_options['title'] . '</div>';
  }
  
  echo '<div id="map'.$id.'" class="mydbr_osm" style="margin-left:auto;margin-right:auto;border:thin solid #bbb; width:'.$width.'; height:'.$height.'"></div></div>'.
    "<script>
  if (!window.hasOwnProperty('OSM_maps') || OSM_maps === undefined) { var OSM_maps = []; }
  var ext_osm_map=OSM_map.initMap('$id',$map_options,$zoom,$heatmap,$heatmap_options);";
  
  $notFound = array();
  if (sizeof($polygons)>0) {
    for ($i=0; $i < sizeof($polygons['latlngs']); $i++) { 
      echo 'OSM_map.newPolygon('.$polygons['latlngs'][$i].','.(isset($polygons['options'][$i]) ? $polygons['options'][$i] : '{}').');';
    }
  }
  for ($i=0; $i<sizeof($dataIn) ; $i++) {
    $links = Extension::get_report_links($dataIn[$i], null, true);
    $bubble = '';
    $heatmap_weight = null;
    $marker_options = '{}';
    if ($mode=='address' || $mode=='osm_node') {
      $lat_lng = $osm->get_coordinates($dataIn[$i][0], $mode);
      if (isset($dataIn[$i][1])) {
        $bubble = $dataIn[$i][1];
      }
      if (isset($dataIn[$i][2])) {
        $marker_options = $dataIn[$i][2];
      }
      if (is_array($lat_lng) && sizeof($lat_lng)==2) {
        $lat = $lat_lng[0];
        $long = $lat_lng[1];
      } else {
        array_push($notFound, 'Missing address for '.$dataIn[$i][0]);
        continue;
      }
    } elseif ($mode=='coordinates_address') {
      if (!is_null($dataIn[$i][0])) {
        $lat = $dataIn[$i][0];
        $long = $dataIn[$i][1];
      } else {
        $lat_lng = $osm->get_coordinates($dataIn[$i][2]);
        if (is_array($lat_lng) && sizeof($lat_lng)==2) {
          $lat = $lat_lng[0];
          $long = $lat_lng[1];
        } else {
          array_push($notFound, 'Missing address for '.$dataIn[$i][2]);
          continue;
        }
      }
      if (isset($dataIn[$i][3])) {
        $bubble = $dataIn[$i][3];
      }
      if (isset($dataIn[$i][4])) {
        $marker_options = $dataIn[$i][4];
      }
    } else {
      $lat = $dataIn[$i][0];
      $long = $dataIn[$i][1];
      if (isset($dataIn[$i][2])) {
        $bubble = $dataIn[$i][2];
      }
      if (isset($dataIn[$i][3])) {
        $marker_options = $dataIn[$i][3];
      }
    }
    if ($heatmap) {
      if ($bubble) {
        $heatmap_weight = floatval($bubble);
        if (!$heatmap_weight || $heatmap_weight<0) {
          $heatmap_weight = null;
        }
      }
      $bubble = '';
    }
    foreach ($links as $key => $link) {
      if ($link['target']=='new_window') {
        $new_window = 'class="new_window" ';
      } else {
        $new_window = '';
      }
      
      $bubble .= '<br><a '.$new_window.'href="'.$link[1].'">'.htmlentities($link[0]).'</a>';
    }
    if ($marker_options=='') $marker_options='{}';
    echo "OSM_map.newMarker($lat, $long,".json_encode($bubble).','.($heatmap_weight ? $heatmap_weight : "null").','.$marker_options.");";
  }
  if ($geojson) {
    echo 'var gjslayer, gjs_bounds = L.latLngBounds();';
    foreach ($geojson as $key => $geojson_url) {
      $options = '';
      if (isset($geojson_options[$key])) {
        if (_str_starts_with($geojson_options[$key], '{')) {
          $options = ','.$geojson_options[$key];
        } else {
          if ($geojson_options[$key]) {
            $options = ',{onEachFeature:'.$geojson_options[$key].'}';
          }
        }
      }
      echo 'gjslayer = L.geoJSON('.file_get_contents($geojson_url).$options.').addTo(ext_osm_map);'."\n".
        'gjs_bounds.extend(gjslayer.getBounds());'."\n";
    }
  }
  echo 'OSM_map.setTiles('.json_encode($tiles).','.$tiles_options.');',
        "for (var i=OSM_maps.length-1; i >=0  ; i--) { 
          if (OSM_maps[i].id=='$id') {
            OSM_maps.splice(i, 1);
          }
        }",
       "OSM_maps.push({'map':Object.assign({}, OSM_map),'mapobj': ext_osm_map ,'id':'$id', 'bounds': ".(is_null($geojson) ? 'null' : 'gjs_bounds')." });",
       'if (OSM_maps.length==1) {$(document).ready(function() { OSM_map.showMaps();});}</script>';
  if (sizeof($notFound)>0) {
    echo '<div class="comment">OpenStreetMap was not able to locate following addresses:<br>';
    foreach ($notFound as $missing) {
      echo '- '.$missing.'<br>';
    }
    echo '</div>';
  }
  
}
