function editor_tab_init()
{
  return { text: '', pos: {left:0, top:0}, selections: editor.listSelections() };
}

function editor_tab_close( tab )
{ 
  var id;
  if (typeof tab == 'undefined') {
    tab = $('#tab_browsing_list li.selected');
  }
  id = tab.attr('data-id');
  if (id == $('#tab_browsing_list li.selected').attr('data-id')) {
    editor_tab_select( tab.prev() );
  }
  tab.remove();
  delete buffers[id];
  if ($('#tab_browsing_list li').length===0) {
    editor.setValue('');
    $('#editor').css('display','none');
  }
  if ($('body').hasClass('proceditor')) {
    $('#editor_result_t'+id).remove();
  }
  
  return false;
}

function editor_get_mode_type_from_ext(ext)
{
  type = '';
  if ($.inArray(ext, ['css', 'php', 'js', 'xml', 'sql', 'html', 'txt', 'tmpl']) >=0 ) {
    type = 'file';
    if (ext=='txt') {
      ext = 'text/plain';
    }
  }
  if (ext==='js') {
    type = 'file';
    ext = 'application/javascript';
  }
  if (ext==='xml') {
    type = 'file';
    ext = 'text/html';
  }
  if (ext==='json') {
    type = 'file';
    ext = 'application/json';
  }
  if (ext=='csv') {
    type = 'file';
    ext = 'text/x-mathematica';
  }
  if ($.inArray(ext, ['eot', 'ttf', 'woff', 'woff2']) >=0 ) {
    type = 'font';
  }
  if ($.inArray(ext, ['jpg', 'png', 'gif', 'jpeg', 'svg', 'webp']) >=0 ) {
    type = 'image';
  }
  
  if (type==='') {
    type = 'unknown';
    ext = 'text/plain';
  }

  return [type, ext];
}

function editor_tab_select( tab ) {
  var id, si, old_tab = $('#tab_browsing_list li.selected'), old_id = old_tab.attr('data-id');
  id = tab.attr('data-id');
  // buffers[id] so that close double click will not happen
  if (old_id!=id && buffers[id]) {
    if (typeof old_id !== "undefined") {
      si = editor.getScrollInfo();
      buffers[old_id].text = editor.getValue();
      buffers[old_id].pos.left = si.left;
      buffers[old_id].pos.top = si.top;
      buffers[old_id].selections = editor.listSelections();
    }
    editor.setValue(buffers[id].text);
    editor.scrollTo(buffers[id].pos.left, buffers[id].pos.top);
    if (buffers[id].selections !== null) {
      editor.setSelections(buffers[id].selections);
    }
    tab.addClass('selected');
    old_tab.removeClass('selected');
    editor.focus();
    if ($('body').hasClass('fileedit')) {
      var ext = tab.attr('data-filename').split('.').pop();
      var type_mode = editor_get_mode_type_from_ext( ext );
      var mode = type_mode[1];
      if (mode=='html') {
        mode = mixedMode;
      }
      editor.setOption("mode", mode);
    }
    if ($('body').hasClass('proceditor')) {
      $('.editor_result_tab:not(#editor_result_t'+id+')').hide();
      $('#editor_result_t'+id).show();
      $('#selected_db').val(tab.attr('data-db'));
    }
  }
}

function editor_tab_set_text( id, text )
{
  buffers[id].text = text;
  buffers[id].pos.left = 0;
  buffers[id].pos.top = 0;
  buffers[id].selections = null;
}

function editor_tab_get_text( id )
{
  var cur_id = $('#tab_browsing_list li.selected').attr('data-id');
  if (parseInt(cur_id, 10)==id) {
    return editor.getValue();
  } else {
    return buffers[id].text;
  }
}


function editor_tab_new( title, text, filename ) 
{
  var id = 1, id_li, $tbl = $('#tab_browsing_list'), title_attr='';

  if (typeof title === "undefined") {
    title = '';
  }
  if (typeof text === "undefined") {
    text = '';
  }
  if (typeof filename === "undefined") {
    filename = '';
  } else {
    title_attr = filename;
    filename = ' data-filename="'+filename+'"';
  }
  
  
  $tbl.children().each( function() { 
    if ($(this).attr('data-id')) {
      id_li = parseInt($(this).attr('data-id'), 10);
      if (id_li > id) id = id_li;
    }
  });
  if ($tbl.children().length===0) {
    id = 1;
  } else {
    id++;
  }
  if (title==='') {
    title = mydbr_loc.MYDBR_SQLEDIT_QUERY+' '+(id-1);
  } 
  $tbl.append('<li title="'+title_attr+'" id="tab_file_query'+id+'" data-id="'+id+'"'+filename+'><div class="tab"><div>'+title+'<div class="close hidden"></div></div></div></li>');
  $('#tab_file_query'+id).attr('data-db', $('#selected_db').val());
  buffers[id] = editor_tab_init();
  editor_tab_select( $tbl.children().last() );
  editor.setValue(text);
  
  if ($('body').hasClass('proceditor')) {
    $('.editor_result_tab').hide();
    $('#editor_results').append('<div id="editor_result_t'+id+'" class="editor_result_tab">'+mydbr_loc.MYDBR_SQLEDIT_NOQUERY+'</div>');
    
  }
}

function editor_gotoline()
{
  var in_dlg = $('#editor_gotoline_dlg').parent('.ui-dialog');
  if ($('#editor_gotoline_dlg').css('display')=='none' || (in_dlg.length>0 && in_dlg.css('display')=='none')) {
    $('#editor_gotoline_dlg').dialog({width: 200, height: 115, resizable: false});
    $('#editor_gotoline_dlg .gotoline').select();
  } else {
    $('#editor_gotoline_dlg .gotoline').focus().select();
  }
}
function editor_gotoline_do()
{
  var line = parseInt($('#editor_gotoline_dlg .gotoline').val(), 10);
  if (line>0) {
    $('#editor_gotoline_dlg').closest('.ui-dialog-content').dialog('close');
    editor.setCursor(line-1, 0);
  } else {
    $('#editor_gotoline_dlg .gotoline').effect("highlight", {color: 'red'}, 250);
    $('#editor_gotoline_dlg .gotoline').focus();
  }
}

function editor_search_find()
{
  var in_dlg = $('#editor_search_dlg').parent('.ui-dialog');
  if ($('#editor_search_dlg').css('display')=='none' || (in_dlg.length>0 && in_dlg.css('display')=='none')) {
    $('#editor_search_dlg').dialog({width: 'auto', height: 'auto', resizable: false});
    $('#editor_search_dlg .searchit').select();
  } else {
    $('#editor_search_dlg .searchit').focus().select();
  }
}

function editor_search_find_do(e)
{
  var pos, 
  found, 
  insensitive = $('#editor_search_match_case').prop('checked'), 
  c = editor.getCursor();

  pos = e.shiftKey ? editor.listSelections()[0].anchor : editor.getCursor();

  window.editor_search = editor.getSearchCursor( $('#editor_search_dlg .searchit').val(), pos, insensitive );

  found = e.shiftKey ? window.editor_search.findPrevious() : window.editor_search.findNext();
  
  if (found) {
    editor.setSelection( window.editor_search.from(), window.editor_search.to() );
    // $('#editor_search_dlg .searchit').focus();
  }
}
$(document).ready(function() {
  $(document).on('click', '#editor_search_dlg .searchb', function(e) {
    editor_search_find_do(e);
  });
  $(document).on('click', '#editor_gotoline_dlg .gotolineb', function(e) {
    editor_gotoline_do();
  });
  $(document).on('keyup', '#editor_gotoline_dlg input.gotoline', function(e) {
    if (e.keyCode == 13) {
      editor_gotoline_do();
    }
  });
  $(document).on('keydown', '#editor_search_dlg .searchit, #editor_search_dlg .replaceit, #editor_search_match_case', function(e) {
    if (e.keyCode==13) {
      if ((!window.editor_prefs.editor_find_with_enter && e.altKey) || (window.editor_prefs.editor_find_with_enter && !e.altKey)) {
        editor_search_find_do(e);
        e.preventDefault();
      }
    }
  });
  
  
  $(document).on('click', '#editor_search_dlg .replaceb', function(e) {
    var insensitive, s = editor.getSelection();
    if (s!=='') {
      editor.replaceSelection($('#editor_search_dlg .replaceit').val());
    }
    insensitive = $('#editor_search_match_case').prop('checked');
    window.editor_search = editor.getSearchCursor( $('#editor_search_dlg .searchit').val(), editor.getCursor(), insensitive );
    if (window.editor_search.findNext()) {
      editor.setSelection( window.editor_search.from(), window.editor_search.to() );
      editor.focus();
    }
  });
  $(document).on('click', '#editor_search_dlg .replaceallb', function(e) {
    var insensitive, s = editor.getSelection();
    if (s!=='') {
      editor.replaceSelection($('#editor_search_dlg .replaceit').val());
    } else {
      s = $('#editor_search_dlg .searchit').val();
    }
    insensitive = $('#editor_search_match_case').prop('checked');
    window.editor_search = editor.getSearchCursor( $('#editor_search_dlg .searchit').val(), editor.getCursor(), insensitive );
    while (window.editor_search.findNext()) {
      editor.setSelection( window.editor_search.from(), window.editor_search.to() );
      if (s!=='') {
        editor.replaceSelection($('#editor_search_dlg .replaceit').val());
      }
    }
    editor.focus();
  });
});

